from math import pi, cos

EARTH = 6378.137  # Radius of the earth in kilometer
METER = (1 / ((2 * pi / 360) * EARTH)) / 1000  # 1 meter in degree

def calculate_corners(lat, lng, area_offset):
    """Calculate corners of polygon based on a center coordinate

    Arguments:
    lat -- center latitude
    lng -- center longitude
    """
    # From https://stackoverflow.com/questions/7477003/calculating-new-longitude-latitude-from-old-n-meters
    # Formulas:
    lat_pos = lat + (area_offset * METER)
    lng_pos = lng + (area_offset * METER) / cos(lat * (pi / 180))

    lat_neg = lat - (area_offset * METER)
    lng_neg = lng - (area_offset * METER) / cos(lat * (pi / 180))

    return [
        (lat_neg, lng_pos), # top left
        (lat_pos, lng_pos), # top right
        (lat_pos, lng_neg), # bottom right
        (lat_neg, lng_neg)  # bottom left
    ]

# separate the zones into smaller area, into a grid
def define_gridareas(lat, lng, area_offset, grid_size):
    # container for an area turned into a grid of areas
    grided_area = []

    # find the main area's corner positions
    main_area = calculate_corners(lat, lng, area_offset)
    # find the main area's range size
    area_size = (main_area[2][0] - main_area[0][0], main_area[0][1] - main_area[2][1])
    # find each subareas vector to it's center
    dist_to_subcenter = (area_size[0]/(grid_size*2), area_size[1]/(grid_size*2))

    # find subareas size relative to main area
    subarea_offset = area_offset/grid_size

    group_id = 0
    i=0
    # find the center coordinates of each area in grid to find the corner areas
    for y in (range(grid_size)):
        relative_size_lng = y / grid_size
        for x in (range(grid_size)):
            relative_size_lat = x / grid_size
            lat_pos = main_area[3][0] + relative_size_lat * area_size[0] + dist_to_subcenter[0]
            lng_pos = main_area[3][1] + relative_size_lng * area_size[1] + dist_to_subcenter[1]
            # id of each subarea
            sub_id = x % grid_size + grid_size * y

            if (x+(y*grid_size)) % ((grid_size*2) + 1) == grid_size * 2:
                group_id += 2

            if (x+(y*grid_size)) % 2 == 0 and (x+(y*grid_size)) != 0:
                i += 1

            # use the center of sub areas to find the corner of each subarea
            corners = calculate_corners(lat_pos, lng_pos, subarea_offset)
            grided_area.append((sub_id, group_id + i%2, corners))

    return grided_area
#[1,2,2,3,4,5,6,3,4,6,8,9,5,3,5.7,8,5,3]
#print(define_gridareas(-60,10,10000,4))
#print(define_gridareas(3435693.5,6299200.0, 400000,4))